# Create "${name}-key" services for NixOps compatibility
#
# This is built as part of the system profile.
# We must be careful not to access `text` / `keyCommand` / `keyFile` here
#
# Sadly, path units don't automatically deactivate the bound units when
# the key files are deleted, so we use inotifywait in the services' scripts.
#
# <https://github.com/systemd/systemd/issues/3642>

{
  config,
  lib,
  pkgs,
  ...
}:

let
  inherit (lib) mapAttrs' nameValuePair;
in

{
  systemd.paths = mapAttrs' (
    name: value:
    nameValuePair "${name}-key" {
      wantedBy = [ "paths.target" ];
      pathConfig.PathExists = value.path;
    }
  ) config.deployment.keys;

  systemd.services = mapAttrs' (
    name: value:
    nameValuePair "${name}-key" {
      enable = true;
      serviceConfig = {
        TimeoutStartSec = "infinity";
        Restart = "always";
        RestartSec = "100ms";
      };
      path = [ pkgs.inotify-tools ];
      preStart = ''
        (while read f; do if [ "$f" = "${value.name}" ]; then break; fi; done \
          < <(inotifywait -qm --format '%f' -e create,move ${value.destDir}) ) &
        if [[ -e "${value.path}" ]]; then
          echo 'flapped down'
          kill %1
          exit 0
        fi
        wait %1
      '';
      script = ''
        inotifywait -qq -e delete_self "${value.path}" &
        if [[ ! -e "${value.path}" ]]; then
          echo 'flapped up'
          exit 0
        fi
        wait %1
      '';
    }
  ) config.deployment.keys;
}
