{
  config,
  lib,
  name,
  ...
}:

let
  inherit (lib) mkOption;

  inherit (lib.types)
    addCheck
    enum
    listOf
    nullOr
    path
    str
    ;
in

{
  options = {
    name = mkOption {
      description = ''
        File name of the key.
      '';
      default = name;
      type = str;
    };

    text = mkOption {
      description = ''
        Content of the key.
        One of `text`, `keyCommand` and `keyFile` must be set.
      '';
      default = null;
      type = nullOr str;
    };

    keyFile = mkOption {
      description = ''
        Path of the local file to read the key from.
        One of `text`, `keyCommand` and `keyFile` must be set.
      '';
      default = null;
      apply = value: if value == null then null else builtins.toString value;
      type = nullOr path;
    };

    keyCommand = mkOption {
      description = ''
        Command to run to generate the key.
        One of `text`, `keyCommand` and `keyFile` must be set.
      '';
      default = null;
      type =
        let
          nonEmptyList = addCheck (listOf str) (l: builtins.length l > 0);
        in
        nullOr nonEmptyList;
    };

    destDir = mkOption {
      description = ''
        Destination directory on the host.
      '';
      default = "/run/keys";
      type = path;
    };

    path = mkOption {
      description = ''
        Full path to the destination.
      '';
      default = "${config.destDir}/${config.name}";
      type = path;
      internal = true;
    };

    user = mkOption {
      description = ''
        The group that will own the file.
      '';
      default = "root";
      type = str;
    };

    group = mkOption {
      description = ''
        The group that will own the file.
      '';
      default = "root";
      type = str;
    };

    permissions = mkOption {
      description = ''
        Permissions to set for the file.
      '';
      default = "0600";
      type = str;
    };

    uploadAt = mkOption {
      description = ''
        When to upload the keys.

        - pre-activation (default): Upload the keys before activating the new system profile.
        - post-activation: Upload the keys after successfully activating the new system profile.

        For `colmena upload-keys`, all keys are uploaded at the same time regardless of the configuration here.
      '';
      default = "pre-activation";
      type = enum [
        "pre-activation"
        "post-activation"
      ];
    };
  };
}
