{ lib, ... }:

let
  inherit (lib) mkOption;

  inherit (lib.types)
    attrsOf
    bool
    nullOr
    path
    str
    unspecified
    ;
in

{
  options = {
    name = mkOption {
      description = ''
        The name of the configuration.
      '';
      type = str;
      default = "hive";
    };

    description = mkOption {
      description = ''
        A short description for the configuration.
      '';
      type = str;
      default = "A Colmena Hive";
    };

    nixpkgs = mkOption {
      description = ''
        The pinned Nixpkgs package set. Accepts one of the following:

        - A path to a Nixpkgs checkout
        - The Nixpkgs lambda (e.g., import <nixpkgs>)
        - An initialized Nixpkgs attribute set

        This option must be specified when using Flakes.
      '';
      type = unspecified;
      default = null;
    };

    nodeNixpkgs = mkOption {
      description = ''
        Node-specific Nixpkgs pins.
      '';
      type = attrsOf unspecified;
      default = { };
    };

    nodeSpecialArgs = mkOption {
      description = ''
        Node-specific special args.
      '';
      type = attrsOf unspecified;
      default = { };
    };

    machinesFile = mkOption {
      description = ''
        Use the machines listed in this file when building this hive configuration.

        If your Colmena host has nix configured to allow for remote builds
        (for nix-daemon, your user being included in trusted-users)
        you can set a machines file that will be passed to the underlying
        nix-store command during derivation realization as a builders option.
        For example, if you support multiple orginizations each with their own
        build machine(s) you can ensure that builds only take place on your
        local machine and/or the machines specified in this file.

        See https://nixos.org/manual/nix/stable/advanced-topics/distributed-builds
        for the machine specification format.

        This option is ignored when builds are initiated on the remote nodes
        themselves via `deployment.buildOnTarget` or `--build-on-target`. To
        still use the Nix distributed build functionality, configure the
        builders on the target nodes with `nix.buildMachines`.
      '';
      default = null;
      apply = value: if value == null then null else builtins.toString value;
      type = nullOr path;
    };

    specialArgs = mkOption {
      description = ''
        A set of special arguments to be passed to NixOS modules.

        This will be merged into the `specialArgs` used to evaluate
        the NixOS configurations.
      '';
      default = { };
      type = attrsOf unspecified;
    };

    allowApplyAll = mkOption {
      description = ''
        Whether to allow deployments without a node filter set.

        If set to false, a node filter must be specified with `--on` when
        deploying.

        It helps prevent accidental deployments to the entire cluster
        when tags are used (e.g., `@production` and `@staging`).
      '';
      default = true;
      type = bool;
    };
  };
}
