{ lib, ... }:

let
  inherit (lib) mkOption;

  inherit (lib.types)
    attrsOf
    functionTo
    submodule
    unspecified
    ;
in

{
  options.registry = mkOption {
    description = ''
      A registry of all system types.
    '';
    type = attrsOf (
      submodule (
        { name, ... }:
        {
          options = {
            evalConfig = mkOption {
              description = ''
                Evaluation function which share the same interface as `nixos/lib/eval-config.nix`
                which can be tailored to your own usecases or to target another type of system,
                e.g. nix-darwin.
              '';
              type = functionTo unspecified;
            };
            supportsDeployment = mkOption {
              description = ''
                Whether this system type supports deployment or not.

                If it supports deployment, it needs to have appropriate activation code,
                refer to how to write custom activators.
              '';
              default = name == "nixos";
              defaultText = "If a NixOS system, then true, otherwise false by default";
            };
            defaults = mkOption {
              description = ''
                Default configuration for that system type.
              '';
              type = functionTo unspecified;
              default = _: { };
            };
          };
        }
      )
    );
  };
}
